/*-----------------------------------------------------------------------
Nom du fichier: RBELEVE.c
Auteur: Philippe MARIANO

Projet: PPE Robot suiveur de ligne
Rle: Fichier source du programme "Elve"  
Cr le: 18/02/2010
Dernire rvision le: 22/02/2011

Cross compilateur utilis: CodeVisionAVR V2
Composant cible : ATMEL 8 bits sur carte "Robot SISI"

Remarque: Ce fichier est destin  un projet CVAVR utilisant la carte SISI
Il doit tre plac dans le  rpertoire du projet 
------------------------------------------------------------------------*/
//============================================================================================================
//============================================================================================================
// Bibliothques   
//============================================================================================================ 
// Bibliothques CVAVR
//------------------------------------------------------------------------------------------------------------
#include <mega32.h>
#include <delay.h>
//------------------------------------------------------------------------------------------------------------
// Bibliothques dveloppes pour la carte SISI et le projet PPE ROBOT SUIVEUR DE LIGNE
//------------------------------------------------------------------------------------------------------------
#include "RB220.h"          // Paramtrage de la programmation                              
#include "RBELEVE.h"          
#include "SISI2SVL.h"       // Contient les fonctions: Lire_BP_I2C_Soft, Affiche_LCD, Arret_Moteurs,  
                            // Active_Moteurs, Affiche_Fleche, AcquerirPositionRobotSVL, 
                            // AfficherPositionRobotSVL 
#include "PCF8574.h"        // Contient les fonctions: Ecrire_Port_I2C_Soft, Lire_Port_I2C_Soft 
//============================================================================================================

//============================================================================================================
// Dclaration des constantes
//============================================================================================================
// EtatCommandeMoteur : diffrents tats possibles pour la commande des moteurs   (NE PAS MODIFIER)
// -----------------------------------------------------------------------------------------------------------
#if FS12 == SOLUTIONELEVE
enum EtatCommandeMoteur EtatCdeMot = DEPART;
#endif
//------------------------------------------------------------------------------------------------------------

// Table de transcodage %PWM --> valeur  placer dans OCR1x pour le contrle des moteurs
//------------------------------------------------------------------------------------------------------------
/* Pour info: Rapports cycliques alpha du signal de commande des moteurs       // A supprimer dans fichier lve
//------------------------------------------------------------------------------------------------------------
// ATTENTION, dans le cas prsent: OCR1x = 255*alpha
//------------------------------------------------------------------------------------------------------------
// 				OCR1x		 	alpha
#define _0PC	 0		 		// 0%
#define _5PC	 13       		// 5%
#define _10PC	 25             // 10%
#define _15PC	 38             // 15%
#define _20PC    51		 		// 20%
#define _25PC	 64             // 25%
#define _30PC    76		 		// 30%
#define _35PC	 89             // 35%
#define _40PC    102		 	// 40%
#define _45PC	 115            // 45%
#define _50PC    128            // 50%
#define _55PC	 140            // 55%
#define _60PC    153            // 60%
#define _65PC	 166            // 65%
#define _70PC    179            // 70%
#define _75PC	 191            // 75%
#define _80PC    205            // 80%
#define _85PC    219            // 85%
#define _90PC    233            // 90%
#define _100%	 253			// ~100%
------------------------------------------------------------------------------------------------------------*/
// A COMPLETER PAR ELEVE en fonction des rsultats de la fiche guide FG4 + les valeurs x5%  (1< x <9)
// ----------------------------------------------------------------------------------------------------------
#if FS12 == SOLUTIONELEVE
const unsigned char Table_Calcul_OCR1x[21][2]={{0,0},{5,13},{10,25},{15,38},{20,51},{25,64},{30,76},
{35,89},{40,102},{45,115},{50,128},{55,140},{60,153},{65,166},{70,179},{75,191},{80,205},{85,219},
{90,233},{95,233},{100,253}};
#endif
//============================================================================================================
//============================================================================================================


//============================================================================================================
//============================================================================================================
// ****************************** SOUS-PROGRAMME SUIVI DE LA LIGNE (ELEVE) ***********************************
//============================================================================================================
//============================================================================================================
// 	Nom : Mode_Run()    Action :  LE ROBOT EFFECTUE SON PARCOURS !
//------------------------------------------------------------------------------------------------------------
// !!!!!!!!!!!!!!!!!! Attention : on sort du menu Mode_Run par une coupure de l'alimentation !!!!!!!!!!!!!!!!!
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Commutateur ON/OFF sur OFF !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//============================================================================================================
void Mode_Run(void)
{
//------------------------------------------------------------------------------------------------------------ 
// Variables locales  Mode_Run()  
//------------------------------------------------------------------------------------------------------------
// type			nom						Commentaires
//------------------------------------------------------------------------------------------------------------ 
unsigned char i;                // Compteur de boucle 

unsigned char VisuPosition;		// Contient la valeur transmise aux LED pendant le dplacement du robot
unsigned char Cde_Del_IR;		// Permet de multiplexer la commande des LED de capteurs d'infrarouges
unsigned char PositionRobot;    // Contient le rsultat du calcul de la position du robot par rapport  la
                                // ligne
#if PARCOURS_ROBOT == AVEC_ARRET
unsigned int Phase_Arret = 50;	// Compteur de boucle : Autorise la mesure de la position du robot
								// pendant 200ms aprs l'arrt du robot devant la cible 
#endif
//============================================================================================================
 
//============================================================================================================
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BOUCLE INFINIE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//------------------------------------------------------------------------------------------------------------
while (1)
    {// dbut Mode_Run() 
//============================================================================================================ 
// 1 - DETECTION DE LA CIBLE 
//------------------------------------------------------------------------------------------------------------
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
// ATTENTION : A dsactiver dans le fichier RBxxx.h en prcisant SANS_ARRET devant #define PARCOURS_ROBOT 
// si la carte capteur cible n'est pas monte sur le robot 
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//------------------------------------------------------------------------------------------------------------
// Phase d'arrt devant la cible: Mesure de la position et volution du graphe "SUIVI DE LIGNE" pendant 
// [4.3*10E-3*Phase_Arret] puis lancement du sous-programme Prise_Balle()
//------------------------------------------------------------------------------------------------------------ 
#if PARCOURS_ROBOT == AVEC_ARRET
       if (Cible == 1)          // Cible passe  1 lorsque le robot est devant la cible 
        {
          if (Phase_Arret == 0)
                Gestion_Balle(); // Cible <- dans INT0
                else 
                  Phase_Arret--;
        }  
#endif
//============================================================================================================     

/*============================================================================================================
// 2 - SUIVI DE LA LIGNE   
//============================================================================================================
// Cycle FS11 -> FS13 -> FS12
//------------------------------------------------------------------------------------------------------------
// Dbut de FS11: "Calculer la position du robot par rapport  la ligne" 		           
//------------------------------------------------------------------------------------------------------------          
Rle: Calculer une valeur numrique permettant de slectioner la LED IR (metteur IR) devant mettre
un faisceau infrarouge et calculer la valeur numrique reprsentative de la position du robot par
rapport  la ligne.
//------------------------------------------------------------------------------------------------------------
Entre: 	ACO		 		bit    		Associ  la sortie du comparateur analogique (interne au C). 
							   			Reprsentatif de la position d'un capteur IR par rapport  la ligne.
                                        ACO = 1 si le capteur IR activ est sur la ligne sinon ACO = 0
//------------------------------------------------------------------------------------------------------------
Sorties:    Cde_Del_IR 		octet   	Permet de multiplexer la commande des LED IR.
										Comme Cde_Del_IR est transfre dans le port A, on donne ci-dessous le
          								dtail des bits de commande des LED IR.
                                                
					   					Emetteur IR connect  au bit      Remarque 
										----------------------------------------------------         
										E_D_IR		"           PA0     |   Emetteur IR droit
										E_CD_IR     "           PA1     |   Emetteur IR centre droit
										E_C_IR		"           PA2     |	Emetteur IR centre
										E_CG_IR     "	        PA3	    |   Emetteur IR centre Gauche
										E_G_IR 		"           PA4     |	Emetteur IR gauche 
					                                                
			Position_Robot	octet 		Image de la position du robot par rapport  la ligne
										(dtails dans RBELEVE.h)
------------------------------------------------------------------------------------------------------------*/ 
//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------
//Solution prof
//------------------------------------------------------------------------------------------------------------
#if FS11 == SOLUTIONPROF 
      PositionRobot = CalculerValeurCapteurLigne();
#else
//------------------------------------------------------------------------------------------------------------
// Solution lve
//------------------------------------------------------------------------------------------------------------
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! REMPLACER les ??? CI-DESSOUS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//------------------------------------------------------------------------------------------------------------
      PositionRobot = HORS_LIGNE; // Initialisation  0	
      Cde_Del_IR = 1; 

      for (i=0;i<5;i++)  // Dbut Acquisition de la position du robot par rapport  la ligne
       {
        CAPTEUR_IR = ~Cde_Del_IR;
      	delay_us(30);	// Activation d'un capteur  ajuster   (19s par dfaut)
        if (ACO == 1) PositionRobot = PositionRobot | Cde_Del_IR;
        CAPTEUR_IR = 0xFF;
        Cde_Del_IR = Cde_Del_IR << 1 ;
        delay_us(540);	//  ajuster   (540s par dfaut)
       }
#endif 
//------------------------------------------------------------------------------------------------------------
// Fin de FS11
//------------------------------------------------------------------------------------------------------------ 

/*------------------------------------------------------------------------------------------------------------
// Dbut FS13 "Choisir la valeur  afficher sur les LED" (Visualiser la position du robot)				   		
//------------------------------------------------------------------------------------------------------------ 
Rle: Choisir la valeur numrique ncessaire  la commande des LED rouge pour que leur clairage soit 
reprsentatif de la position du robot par rapport  la ligne noire.
//------------------------------------------------------------------------------------------------------------
Entre 
Position_Robot:     octet 	    Reprsentatif de la position du robot. Les deux positions possibles pour  
                                le capteur "LIGNE" sont traites dans RBELEVE.h (Connecteur du capteur vers 
                                l'avant ou connecteur du capteur vers l'arrire).
//------------------------------------------------------------------------------------------------------------
Sortie
VisuPosition:       octet  	    Reprsentatif des LED  clairer. Le bit 0 de VisuPosition commande la 
						        Led D0 de l'interface IHM etc. Les valeurs  placer dans VisuPosition sont dfinies 
						        dans le fichier RBELEVE.h
------------------------------------------------------------------------------------------------------------*/
//------------------------------------------------------------------------------------------------------------
//Solution prof
//------------------------------------------------------------------------------------------------------------
#if FS13 == SOLUTIONPROF

#if  POSITIONCONNECTEUR == CONNECTEURAR 
        VisuPosition = AfficherPositionRobotSVL(PositionRobot);  
     
#elif POSITIONCONNECTEUR == CONNECTEURAV 
        PositionRobot = AdapterValeurCapteurLigneSiConnecteurAV(PositionRobot);
        VisuPosition = AfficherPositionRobotSVL(PositionRobot);
#endif
//------------------------------------------------------------------------------------------------------------
// Solution lve
//------------------------------------------------------------------------------------------------------------        
#else
//------------------------------------------------------------------------------------------------------------                  
        switch (PositionRobot)              
        {  
//------------------------------------------------------------------------------------------------------------       
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! A vrifier PAR ELEVE  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//------------------------------------------------------------------------------------------------------------        
// 		    Position_Robot 			Modification de la commande des LED             Commentaires    				
//------------------------------------------------------------------------------------------------------------
		    case HORS_LIGNE:  		VisuPosition = Visu_Hors_ligne; break;			// Hors ligne
   		    case GAUCHE_3PLUS:  	VisuPosition = Visu_GAUCHE_3PLUS; break;		// Gauche3+
  		    case GAUCHE_2PLUS:  	VisuPosition = Visu_GAUCHE_2PLUS; break;		// Gauche2+
   		    case CENTRE_1:     		VisuPosition = Visu_CENTRE_1; break;			// Centre1 non visualis
            case GAUCHE_PLUS:  		VisuPosition = Visu_GAUCHE_PLUS; break;			// Gauche+  
            case CENTRE_3: 	   		VisuPosition = Visu_CENTRE_3; break;			// Centre3 
            case GAUCHE: 			VisuPosition = Visu_GAUCHE; break;				// Gauche 
            case DROITE_3PLUS: 		VisuPosition = Visu_DROITE_3PLUS; break;		// Droite3+
            case DROITE_2PLUS: 		VisuPosition = Visu_DROITE_2PLUS; break;		// Droite2+  
            case DROITE_PLUS:  		VisuPosition = Visu_DROITE_PLUS; break;			// Droite+ 
            case DROITE: 	   		VisuPosition = Visu_DROITE; break;				// Droite 
            case CENTRE_5: 			VisuPosition = Visu_CENTRE_5; break;			// Centre5   
            default :				VisuPosition = 0b00011111; break;				// Visu_Erreur_mesure_ligne
        }
//------------------------------------------------------------------------------------------------------------
#endif 
//------------------------------------------------------------------------------------------------------------    
        Ecrire_Port_I2C_Soft(Add_LED_I2C,VisuPosition);  		// Ecriture sur les LED via le bus I2C 
//------------------------------------------------------------------------------------------------------------            
// Fin de FS13   
//------------------------------------------------------------------------------------------------------------


/*------------------------------------------------------------------------------------------------------------ 
// Dbut FS12: "Choisir les valeurs des rapports cycliques"	(Corriger la trajectoire du robot)				
//------------------------------------------------------------------------------------------------------------     
Rle: Choisir la valeur des rapports cycliques des signaux de commande des moteurs du robot en fonction de
sa position par rapport  la ligne noire.
//------------------------------------------------------------------------------------------------------------
Entre:          Position_Robot           octet     (voir FS13)    
//------------------------------------------------------------------------------------------------------------
Sorties:         CDE_MOTD                 octet		Correspond au registre (OCR1A). Rgle le rapport cyclique 
													du signal de commande du moteur droit.
				 CDE_MOTG                 octet     Correspond au registre (OCR1B). Rgle le rapport cyclique
				 							 		du signal de commande du moteur gauche.
------------------------------------------------------------------------------------------------------------*/ 
// Graphe d'tats-transitiond du suivi de ligne
//------------------------------------------------------------------------------------------------------------
//Solution prof
//------------------------------------------------------------------------------------------------------------
#if FS12 == SOLUTIONPROF
        CorrigerTrajectoireRobotSVL(PositionRobot);
    
//------------------------------------------------------------------------------------------------------------
// Solution lve
//------------------------------------------------------------------------------------------------------------
#else
     switch (EtatCdeMot)              
       { // Dbut switch (EtatCdeMot) 
// "Dpart"!                                
        case DEPART: Arret_Moteurs(); 
        		if ((PositionRobot == CENTRE_1)||(PositionRobot == CENTRE_3)||(PositionRobot == CENTRE_5))
        		    { 
        		      Active_Moteurs();
        		      EtatCdeMot = CDE_CENTRE;
        		    }
        		    else  EtatCdeMot = DEPART;
        break;   
//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------       
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!PARTIE A COMPLETER PAR ELEVE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
//------------------------------------------------------------------------------------------------------------
// "Suivi de la ligne"        
        case CDE_CENTRE : CDE_MOTG = PWM[0][0]; CDE_MOTD = PWM[0][1];
                            if (PositionRobot == GAUCHE) EtatCdeMot = CDE_DROITE;
                          	    else if (PositionRobot == DROITE) EtatCdeMot = CDE_GAUCHE;
                          		    else if (PositionRobot == HORS_LIGNE) EtatCdeMot = CDE_COUPURE;
                          			    else EtatCdeMot = CDE_CENTRE;
        break;  
        
        
        case CDE_DROITE:CDE_MOTG = PWM[1][0]; CDE_MOTD = PWM[1][1]; 
                            if (PositionRobot == CENTRE_5 ) EtatCdeMot = CDE_CENTRE;
                          	    else if (PositionRobot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS;
                           		    else if (PositionRobot == HORS_LIGNE) EtatCdeMot = CDE_COUPURE;
                          			    else EtatCdeMot = CDE_DROITE;
        break;   
        
        
                         // !!!!!!!!!!!!!!!!! Champ  renseigner  !!!!!!!!!!!!!!!!!

//------------------------------------------------------------------------------------------------------------        
// Fin deFS12 
//------------------------------------------------------------------------------------------------------------        
       }// Fin switch (EtatCdeMot)   */  
#endif   
//------------------------------------------------------------------------------------------------------------
   
// Fin Traiter 
//============================================================================================================
       delay_us(2000); //  ajuster   (704s  par dfaut) 
//============================================================================================================		
    }
} // fin Mode_Run();
//============================================================================================================   

//============================================================================================================ 
//============================================================================================================
// *************************** SOUS-PROGRAMME GESTION DE LA BALLE ELEVE **************************************
//============================================================================================================
//============================================================================================================

#if PARCOURS_ROBOT == AVEC_ARRET
void Gestion_Balle(void)
{
// Dsactivation de la dtection de la cible
Cible = 0; 
//---------------------------------------------------------
// A complter par lve
//---------------------------------------------------------
//
CDE_SERVO1 = 1; // Libration balle (Le bras se lve)
delay_ms(300);
CDE_SERVO1 = 0; // Arrt commande du bras
CDE_SERVO2 = 1;  // Libration du marteau
delay_ms(1500);  // Attente vacuation de la balle
//---------------------------------------------------------
// Marche Robot 
//---------------------------------------------------------
Active_Moteurs();  
//---------------------------------------------------------
}
#endif

//============================================================================================================